class PHireField
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::Attributes::Dynamic
  include ::AttributeFieldsHelper
  include ::Admin::PHireFieldHelper
  field :display_in_reason_for_hire, type: Boolean, default: false
  field :key, type: String
  field :af_count
  field :title, type: String, localize: true
  field :markup, default: "text_field"
  field :option_list, type: Hash,default: {}
  field :markup_options, type: Hash
  field :built_in, type: Boolean, default: false
  field :disabled, type: Boolean, default: false
  field :to_delete, type: Boolean, default: false
  
  field :to_require,type: Boolean, default: true
  
  field :typeA, type: Hash, default: {cross_lang: false}
  field :typeB, type: Hash, default: {}
  field :typeC, type: Hash, default: {calendar: "west_calendar", format: "format3"}
  field :typeD, type: Hash, default: {cross_lang: false}
  field :typeE, type: Hash, default: {}
  field :typeF, type: Hash, default: {}
  belongs_to :property
  has_many :p_hire_field_values, autosave: true, dependent: :destroy
  accepts_nested_attributes_for :p_hire_field_values, :allow_destroy => true
  before_save :check_option_list
  before_destroy do
    @property = self.property
    if @property && @property.custom_field_names
      @property.custom_field_names.delete("p_hire_fields.#{self.id}")
      @property.save
    end
  end
  after_save do
    unless self.disabled
      @property = self.property
      if @property && @property.custom_field_names
        if self.new_record? || !(@property.custom_field_names.include?("p_hire_fields.#{self.id}"))
          @property.custom_field_names << "p_hire_fields.#{self.id}"
          @property.save
        end
      end
    end
    if self.display_in_reason_for_hire_changed? && self.property
      self.property.update(:need_change_tmp_reason=>true)
    end
  end
  def markup_value
    get_data["option_list"]
  end

  def add_more
     (get_data["add_more"] == "true" ? true : false) rescue false
  end

  def locale
    get_data["cross_lang"] == "true" ? false : true
  end

  def self_defined_markup_options?
    (self.property.method(self[:key].pluralize.to_sym)  && self.property.method(self[:key].pluralize+"_for_"+markup)) rescue false
  end

  def option_list
    if self_defined_markup_options?
      #Class need to have corresponding field and value agent 
     # Ex: For "status" the class must have field called "statuses" for the relation and "statuses_for_select" for the select function 
     method = self.property.method(self[:key].pluralize+"_for_"+markup)
      return  (method.call rescue {})
    elsif self[:option_list].nil?  || (self[:option_list].empty?)
      return {}
    else
      return self[:option_list]
    end
  end

  def markup_options=(var)
    self[:markup_options] = (eval(var)  rescue {})
  end

  def markup_options
    if self[:markup_options].nil?
      return {}
    else
      Hash[self[:markup_options].map{|key,val|[key.to_sym,val]}]  rescue {}
    end

  end

  def panel
    panel  = $property_list[:markups][self[:markup]]["panel"]
  end

  def get_data
    self[panel]
  end
  def get_placeholder(locale=I18n.locale)
    placeholder = self.get_data["placeholder"] rescue nil
    if placeholder
      if placeholder.respond_to?(:keys)
        placeholder[locale.to_s]
      else
        placeholder
      end
    else
      ""
    end
  end
  def typeA=(var)
    check_add_more_convert(var)
    check_cross_lang_convert(var,"typeA")
    self["typeA"] = var
  end

  def typeD=(var)
    check_cross_lang_convert(var,"typeD")
    self["typeD"] = var
  end

  def is_built_in?
    self.built_in
  end
  
  def is_disabled?
    self.disabled
  end

  def self.add_p_hire_field(property,property_param, p_hire_field_id=nil,field_status)
    @field_name = 'property'
    if field_status.eql?(true)
      @p_hire_field_counter = property.p_hire_fields_enabled.count rescue nil
      @p_hire_field = self.find(p_hire_field_id) rescue nil
      old_key = @p_hire_field.key
      @p_hire_field.update(property_param)
      @p_hire_field.save
      attribute_values = @p_hire_field.p_hire_field_values
      if attribute_values.count > 0
        attribute_values.each do |av|
          av.key = property_param["key"]
          av.save
        end
      end
      @p_hire_field[:af_count] = @p_hire_field_counter
    else
      @p_hire_field_counter = property.p_hire_fields_enabled.count rescue nil
      @p_hire_field = property.p_hire_fields.build(property_param) rescue nil
      @p_hire_field.save  
      @p_hire_field[:af_count] = @p_hire_field_counter
    end
    return @p_hire_field
  end

  protected

  def check_cross_lang_convert(var,field)
    if self[field]["cross_lang"] != var["cross_lang"]
      case var["cross_lang"]
      when "true"  #from no-add_more to add_more
        cross_lang_convert(:to_cross_lang)  
      else #from add_more to no-add_more
        cross_lang_convert(:to_no_cross_lang)
      end # of case
    end # of if
  end

   def check_add_more_convert(var)
    if self["typeA"]["add_more"] != var["add_more"]
      case var["add_more"]
      when "true"  #from no-add_more to add_more
        add_more_convert(:to_add_more)  
      else #from add_more to no-add_more
        add_more_convert(:to_no_add_more)
      end # of case
    end # of if
  end

  def cross_lang_convert(opt) 
    
  end

  def check_option_list
    self[:option_list] = self[panel]["option_list"] rescue nil
  end

  def add_more_convert(opt) 

  end

end