class PHireFieldValue
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::Attributes::Dynamic
  include ::AttributeValuesHelper
  include ::Admin::PHireFieldValuesHelper

  field :key, type: String

  belongs_to :p_hire_field
  belongs_to :p_hire

  before_save :check_key
  before_save :data_proc

  def add_more_counter
    index_max = self["val"].count rescue 0
    index_max == 0 ?   1 : index_max
  end

  def value(index = nil)
  	site = Site.first
    result = case self.p_hire_field.markup
      when 'text_field','text_area'
        if self.p_hire_field.add_more and (self.p_hire_field.markup == "text_field")
          index.nil? ? self["val"] : self["val"][index]
        else
          self.p_hire_field.get_data["cross_lang"] =="true" ?  self["val"] : Hash[site.valid_locales.collect{|lang|  [lang,self[lang.to_sym]]}]
        end
      when 'select','radio_button','address'
        self["val"]
      when 'date'
          if !self["val"].blank? and !self["val"]['(1i)'].blank?
            "#{self["val"]['(1i)']}/#{self["val"]['(2i)']}/#{self["val"]['(3i)']}"
          else
            self["val"]
          end
      when 'checkbox'
        self["val"]
    end #end of case self.p_hire_field.markup
  end

  def value=(value)
    #save everything to temp_data waiting for futher process
    self[:temp_data] = value
  end

  def get_field_value
    p_hire_field = self.p_hire_field
    if (p_hire_field.markup.eql?("text_field") || p_hire_field.markup.eql?("text_area"))
      if self.value.class == String
        field_value = self.value
      else
        field_value = self.value[I18n.locale]
      end
    elsif (p_hire_field.markup.eql?("select") || p_hire_field.markup.eql?("radio_button"))
      field_value = p_hire_field.markup_value["#{self.value}"][I18n.locale] rescue nil
    elsif p_hire_field.markup.eql?("address")
      field_value = rf[:address_key][I18n.locale].map{|k,v| v}.delete_if(&:blank?).join(', ')
    elsif p_hire_field.markup.eql?("date")
      case p_hire_field.typeC['format']
      when 'format1'
        field_value = self.value.to_date.strftime("%Y/%m/%d")
      when 'format2'
        field_value = self.value.to_date.strftime("%Y/%m/%d")
      when 'format3'
        field_value = self.value.to_date.strftime("%Y/%m")       
      when 'format4'
        field_value = self.value.to_date.strftime("%Y")  
      end
    elsif p_hire_field.markup.eql?("checkbox")
      field_value = self.value.map {|v| p_hire_field.markup_value["#{v}"][I18n.locale]}.join(', ') rescue nil
    end

    field_value = (field_value =~ /\A#{URI::regexp(['http', 'https'])}\z/) ? "<a href='#{field_value}' target='blank'>#{field_value}</a>" : field_value
    field_value = (field_value =~ /\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b/i) ? "<a href='mailto:#{field_value}'>#{field_value}</a>" : field_value
    if p_hire_field.markup.eql?("hint_text")
      field_value = p_hire_field.get_placeholder rescue ""
      {
        "key" => p_hire_field.key,
        "title" => p_hire_field.title,
        "value" => field_value,
        "val" => field_value,
        "hint" => true
      }
    elsif !field_value.blank?
      {
        "key" => p_hire_field.key,
        "title" => p_hire_field.title,
        "value" => field_value,
        "val" => field_value = self.value
      }
    else
      {
        "key" => p_hire_field.key,
        "title" => p_hire_field.title,
        "value" => "",
        "val" => field_value = self.value
      }
    end
  end

  def get_value_by_locale(locale,add_more_index=nil)
    p_hire_field = self.p_hire_field
    case p_hire_field.markup
      when "text_field"
        case p_hire_field.add_more
        when true
          if p_hire_field.locale
            add_more_index.nil? ? self.value.collect{|t| t[locale.to_s]}.join(",") : self.value(add_more_index)[locale]
          else
            add_more_index.nil? ? self.value.join(",") : self.value(add_more_index)
          end
          
        when false
          p_hire_field.locale ? self[locale.to_s]  : self.value
        end

      when "select"
        markup_values = p_hire_field.self_defined_markup_options? ? p_hire_field.markup_value : self.p_hire_field.markup_value
        markup_values[self.value][locale.to_s] rescue 'NoData'

      when "text_area"
        p_hire_field.locale ? self[locale.to_s]  : self.value

      when "date"
        if p_hire_field.date_is_range?
          get_date_by_format(:from) + ' ~ '  + get_date_by_format(:to)
          # self.value["from"] + ' ~ ' + self.value["to"]
        else
          get_date_by_format  
          # self.value
        end

      when "address"
        self.value[locale.to_s]

      when "radio_button"
        markup_values = p_hire_field.markup_value
        markup_values[self.value][locale.to_s]

      when "checkbox" 
        markup_values = p_hire_field.markup_value
        self.value.collect{|key| markup_values["#{key}"][I18n.locale]}.join(",")
      when "date_durnation"
        self.value

      else
        p_hire_field.locale ? self[locale.to_s]  : self.value
    end
  end

  def get_date(item = nil)
    case item
    when :from
      # data = self[:val]["from"]
      data = self.value["from"]
    when :to
      # data = self[:val]["to"]
      data = self.value["to"]
    when nil
      # data = self[:val]
      data = self.value
    end

    # Date.new(data["(1i)"].to_i,data["(2i)"].to_i,data["(3i)"].to_i) rescue nil
  end

  def self.put_field_values(p_hire, field_value_param, field_value_id=nil,field_value_status)
    if field_value_status.eql?(true)
      @p_hire_field_value = p_hire.p_hire_field_values.find(field_value_id) rescue nil
      if @p_hire_field_value!=nil
        @p_hire_field_value.update(field_value_param) rescue nil
        @p_hire_field_value.save rescue nil
      end
    else
      @p_hire_field_value = p_hire.p_hire_field_values.build(field_value_param) rescue nil
      @p_hire_field_value.save
    end
    return @p_hire_field_value
  end


protected
  
def unset_all_lang_values
    VALID_LOCALES.each{|t| self.unset t}
  end

  def data_proc
    p_hire_field = self.p_hire_field
    unless self[:temp_data].nil?
      case p_hire_field.markup
        when "address"
           self["val"] = self["temp_data"]
        when 'text_field','text_area'
          if p_hire_field.add_more
            self["val"] = self["temp_data"]
          else # if not add_more
            if p_hire_field.can_muti_lang_input?
                self[:temp_data].each do |key,val| 
                  self[key] = val
                  end if(!p_hire_field.get_data[:cross_lang]) 
              else
                self["val"] = self[:temp_data]
            end
          end # of self.p_hire_field.add_more
         
        when 'select','date','radio_button'
          self["val"] = self[:temp_data]
        when 'checkbox'
          self["val"] = self[:temp_data].keys 
      end #end of case self.p_hire_field.markup  
    end # of self[:temp_data].nil?
    self.unset('temp_data')
    self.unset('temp')
  end #of data_proc

  def check_key
    if self.p_hire_field_id.present? && self.p_hire_field.nil?
      begin
        self.p_hire_field = PHireField.find(self.p_hire_field_id)
      rescue
        nil
      end
    end
    self.key = self.p_hire_field.key rescue nil
  end

  def method_missing(*field)
    if field.size < 1
      self[field[0]]  
    else
      self[(field[0].to_s.delete "=")] = field[1]
    end
  end
end
